import { SqliteDialect, SqliteDriver, } from 'kysely';
const CONNECTION_TIMEOUT = 10_000;
class ConnectionMutex {
    #promise;
    #resolve;
    async lock() {
        while (this.#promise) {
            await this.#promise;
        }
        this.#promise = new Promise(resolve => {
            this.#resolve = resolve;
        });
    }
    unlock() {
        const resolve = this.#resolve;
        this.#promise = undefined;
        this.#resolve = undefined;
        resolve?.();
    }
}
class LibSqlConnection {
    db;
    created = Date.now();
    constructor(db) {
        this.db = db;
    }
    isValid() {
        return this.memory || this.created > Date.now() - CONNECTION_TIMEOUT;
    }
    async executeQuery(compiledQuery) {
        const { sql, parameters } = compiledQuery;
        const stmt = this.db.prepare(sql);
        if (stmt.reader) {
            return {
                rows: stmt.all(parameters),
            };
        }
        const query = sql.trim().toLowerCase();
        /* v8 ignore next 5 */
        if (query.startsWith('select') || ((query.startsWith('insert into') || query.startsWith('update ')) && query.includes(' returning '))) {
            return {
                rows: stmt.all(parameters),
            };
        }
        const { changes, lastInsertRowid } = stmt.run(parameters);
        return {
            numAffectedRows: changes,
            insertId: lastInsertRowid,
            rows: [],
        };
    }
    /* v8 ignore start */
    async *streamQuery(compiledQuery) {
        const { sql, parameters } = compiledQuery;
        const stmt = this.db.prepare(sql);
        if (!sql.toLowerCase().startsWith('select')) {
            throw new Error('Sqlite driver only supports streaming of select queries');
        }
        for (const row of stmt.iterate(parameters)) {
            yield {
                rows: [row],
            };
        }
    }
}
class LibSqlKyselyDriver extends SqliteDriver {
    config;
    db;
    connection;
    connectionMutex = new ConnectionMutex();
    constructor(config) {
        super(config);
        this.config = config;
    }
    async init() {
        this.db = await this.config.database();
        this.connection = new LibSqlConnection(this.db);
        /* v8 ignore next 3 */
        if (this.config.onCreateConnection) {
            await this.config.onCreateConnection(this.connection);
        }
    }
    async acquireConnection() {
        await this.connectionMutex.lock();
        /* v8 ignore next 4 */
        if (!this.connection.isValid()) {
            await this.destroy();
            await this.init();
        }
        return this.connection;
    }
    async releaseConnection() {
        this.connectionMutex.unlock();
    }
    async destroy() {
        this.db.close();
    }
}
export class LibSqlDialect extends SqliteDialect {
    config;
    constructor(config) {
        super(config);
        this.config = config;
    }
    createDriver() {
        return new LibSqlKyselyDriver(this.config);
    }
}
